#
# 3DE4.script.name:     Katana Renderman bs_tdeLens...
#
# 3DE4.script.version:  v1.1
#
# 3DE4.script.gui:      Main Window::3DE4::Export Project
#
# 3DE4.script.comment:  Exports Lens distortion data from current lens to Katana Python script that sets Renderman PrmanGlobalSettings (bs_tdeLens for Katana should be installed to read the distortion data)
#
#

#
# import sdv's python vector lib...

from vl_sdv import *

#
# functions...

def convertToAngles(r3d):
    rot	= rot3d(mat3d(r3d)).angles(VL_APPLY_ZXY)
    rx	= (rot[0]*180.0)/3.141592654
    ry	= (rot[1]*180.0)/3.141592654
    rz	= (rot[2]*180.0)/3.141592654
    return(rx,ry,rz)


def convertZup(p3d,yup):
    if yup==1:
        return(p3d)
    else:
        return([p3d[0],-p3d[2],p3d[1]])


def angleMod360(d0,d):
    dd	= d-d0
    if dd>180.0:
        d	= angleMod360(d0,d-360.0)
    else:
        if dd<-180.0:
            d	= angleMod360(d0,d+360.0)
    return d


def validName(name):
    name	= name.replace(" ","_")
    name	= name.replace("\n","")
    name	= name.replace("\r","")
    return name

def prepareImagePath(path,startframe):
    path	= path.replace("\\","/")
    i	= 0
    n	= 0
    i0	= -1
    while(i<len(path)):
        if path[i]=='#': n += 1
        if n==1: i0 = i
        i	+= 1
    if i0!=-1:
        fstring		= "%%s%%0%dd%%s"%(n)
        path2		= fstring%(path[0:i0],startframe,path[i0+n:len(path)])
        path		= path2
    return path


def validName(name):
    name    = name.replace(" ","_")
    name    = name.replace("\n","")
    name    = name.replace("\r","")
    return name


#
# search for camera point group...

campg   = None
pgl     = tde4.getPGroupList()

for pg in pgl:
    if tde4.getPGroupType(pg)=="CAMERA":
        campg = pg

if campg==None:
    tde4.postQuestionRequester("Export Nothing Real bs_tdeLens RMS...","Error, there is no camera point group.","Ok")

#
# open requester...

try:
    req     = _export_requester_katana
except (ValueError,NameError,TypeError):
    _export_requester_katana  = tde4.createCustomRequester()
    req                     = _export_requester_katana
    tde4.addFileWidget(req,"file_browser","Exportfile...","*.py")
    tde4.addTextFieldWidget(req, "startframe_field", "Startframe", "1")
    # tde4.addOptionMenuWidget(req,"mode_menu","Orientation","Y-Up", "Z-Up")
    tde4.addToggleWidget(req,"hide_ref_frames","Hide Reference Frames",0)
ret     = tde4.postCustomRequester(req,"Export Nothing Real bs_tdeLens RMS_Katana (Python-Script)...",600,0,"Ok","Cancel")
if ret==1:
    # yup   = tde4.getWidgetValue(req,"mode_menu")
    # if yup==2: yup = 0
    yup     = 1
    path    = tde4.getWidgetValue(req,"file_browser")
    frame0  = float(tde4.getWidgetValue(req,"startframe_field"))
    frame0  -= 1
    hide_ref= tde4.getWidgetValue(req,"hide_ref_frames")
    if path!=None:
        if not path.endswith('.py'): path = path+'.py' 
        f       = open(path,"w")
        if not f.closed:

            # Prepare camera information                        
            cl      = tde4.getCameraList()
            index   = 1
            for cam in cl:
                camType         = tde4.getCameraType(cam)
                noframes        = tde4.getCameraNoFrames(cam)
                lens            = tde4.getCameraLens(cam)
                if lens!=None:
                    name            = validName(tde4.getCameraName(cam))
                    name            = "%s_%s_1"%(name,index)
                    index           += 1
                    fback_w         = tde4.getLensFBackWidth(lens)
                    fback_h         = tde4.getLensFBackHeight(lens)
                    p_aspect        = tde4.getLensPixelAspect(lens)
                    focal           = tde4.getCameraFocalLength(cam,1)
                    lco_x           = tde4.getLensLensCenterX(lens)
                    lco_y           = tde4.getLensLensCenterY(lens)
                    
                    # convert filmback to inch...
                    fback_w         = fback_w/2.54
                    fback_h         = fback_h/2.54
                    lco_x           = -lco_x/2.54
                    lco_y           = -lco_y/2.54
                    screenAspect    = (fback_h/p_aspect)/fback_w    # taking in account the pixel aspect when calculating screen window height
                    
                    # convert focal length to mm...
                    focal           = focal*10.0
                    
                    model = tde4.getLensLDModel(lens)
                    
                    # Set Renderman globals camera lens information:
                    f.write(   'def setChildParamsDefault( parameter ):\n')
                    f.write(   '    children = parameter.getChildren()\n')
                    f.write(   '    if children :\n')
                    f.write(   '        childDict = dict( (child.getName(),child) for child in children )\n')
                    f.write(   '        if "enable" in childDict and "value" in childDict and "type" in childDict :\n')
                    f.write(   '                childDict[ "enable" ].setValue( 0, 0 )\n')
                    f.write(   '        else :\n')
                    f.write(   '                [ setChildParamsDefault( child ) for child in childDict.itervalues() ]\n\n')

                    f.write(   'def setLocal(param):\n')
                    f.write(   '    param.getChild("enable").setValue(1, 0)\n\n')

                    f.write(   'node=NodegraphAPI.GetAllSelectedNodes()\n')
                    f.write(   'try:\n')
                    f.write(   '    rman=node[0]\n')
                    f.write(   'except:\n')
                    f.write(   '    print "Please select the \'PrmanGlobalStatements\' node you want to configure to render with bs_tdeLens"\n\n')
                    f.write(   'if node[0].getType() == "PrmanGlobalStatements":\n')
                    f.write(   "    rman = rman.getParameter('prmanGlobalStatements.camera.projection')\n")
                    f.write(   '    setChildParamsDefault( rman )\n')
                    '''
                    '''

                    # Writing the static lens params:
                    f.write(   '    projType = rman.getChild("projectionType")\n')
                    f.write(   '    projType.getChild("value").setValue("bs_tdeLens", 0)\n')
                    f.write(   '    setLocal(projType)\n')
                    
                    '''
                    we calculate the pixel aspect inside the lens code
                    f.write(   '    projType = rman.getChild("pixel_aspect")\n')
                    f.write(   '    projType.getChild("value").setValue(%s, 0)\n'%p_aspect)
                    f.write(   '    setLocal(projType)\n')
                    '''
                    if model == '3DE Classic LD Model':
                        f.write(   '    param = rman.getChild("model")\n')
                        f.write(   '    param.getChild("value").setValue(0, 0)\n')
                        
                    elif model == '3DE4 Radial - Standard, Degree 4':
                        f.write(   '    param = rman.getChild("model")\n')
                        f.write(   '    param.getChild("value").setValue(1, 0)\n')
                        
                    elif model == '3DE4 Anamorphic - Standard, Degree 4':
                        f.write(   '    param = rman.getChild("model")\n')
                        f.write(   '    param.getChild("value").setValue(2, 0)\n')
                        
                    elif model == '3DE4 Radial - Fisheye, Degree 8':
                        f.write(   '    param = rman.getChild("model")\n')
                        f.write(   '    param.getChild("value").setValue(3, 0)\n')
                    
                    elif model == '3DE4 Anamorphic, Degree 6':
                        f.write(   '    param = rman.getChild("model")\n')
                        f.write(   '    param.getChild("value").setValue(4, 0)\n')
                    f.write(   '    setLocal(param)\n')
                    
                    f.write(   '    param = rman.getChild("filmBack_W")\n')
                    f.write(   '    param.getChild("value").setValue(%s, 0)\n'%fback_w)
                    f.write(   '    setLocal(param)\n')
                    
                    f.write(   '    param = rman.getChild("filmBack_H")\n')
                    f.write(   '    param.getChild("value").setValue(%s, 0)\n'%fback_h)
                    f.write(   '    setLocal(param)\n')
                    
                    f.write(   '    param = rman.getChild("xOffset")\n')
                    f.write(   '    param.getChild("value").setValue(%s, 0)\n'%lco_x)
                    f.write(   '    setLocal(param)\n')
                    
                    f.write(   '    param = rman.getChild("yOffset")\n')
                    f.write(   '    param.getChild("value").setValue(%s, 0)\n'%lco_y)
                    f.write(   '    setLocal(param)\n')
                    
                    # Writing the animated and lens specific params:                
                    frame   = 1
                    while frame<=noframes:
                        # rot/pos...
                        p3d	= tde4.getPGroupPosition3D(campg,cam,frame)
                        p3d	= convertZup(p3d,yup)
                        r3d	= tde4.getPGroupRotation3D(campg,cam,frame)
                        rot	= convertToAngles(r3d)
                        if frame>1:
                            rot	= [ angleMod360(rot0[0],rot[0]), angleMod360(rot0[1],rot[1]), angleMod360(rot0[2],rot[2]) ]
                        rot0	= rot

                        # focal length...
                        F       = tde4.getCameraFocalLength(cam,frame)
                        focal   = F*10.0

                        # Focus Distance...
                        focus   = tde4.getCameraFocus(cam, frame)

                        f.write(   '    param = rman.getChild("focal_length")\n')
                        f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                        f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(focal, frame+frame0) ) 
                        f.write(   '    setLocal(param)\n\n')
                        
                        f.write(   '    param = rman.getChild("focus_distance")\n')
                        f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                        f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(focus, frame+frame0) ) 
                        f.write(   '    setLocal(param)\n\n')
                        
                        # Set Distortion parameters:
                        if model == '3DE Classic LD Model':
                            d  = tde4.getLensDistortion(lens,F)
                            f.write(   '    param = rman.getChild("distortion")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(d, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            sq = tde4.getLensAnamorphSqueeze(lens,F)
                            f.write(   '    param = rman.getChild("anamorphicSqueeze")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(sq, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            xC = tde4.getLensCurvatureX(lens,F)
                            f.write(   '    param = rman.getChild("xCurvature")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(xC, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            yC = tde4.getLensCurvatureY(lens,F)
                            f.write(   '    param = rman.getChild("yCurvature")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(yC, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            q  = tde4.getLensQuarticDistortion(lens,F)
                            f.write(   '    param = rman.getChild("quarticDist")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(q, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                        
                        elif model == '3DE4 Radial - Standard, Degree 4':
                            dd2 = tde4.getLensLDAdjustableParameter(lens, "Distortion - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("dd2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(dd2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            ud2 = tde4.getLensLDAdjustableParameter(lens, "U - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("ud2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(ud2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            vd2 = tde4.getLensLDAdjustableParameter(lens, "V - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("vd2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(vd2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            qd4 = tde4.getLensLDAdjustableParameter(lens, "Quartic Distortion - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("qd4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(qd4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            ud4 = tde4.getLensLDAdjustableParameter(lens, "U - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("ud4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(ud4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            vd4 = tde4.getLensLDAdjustableParameter(lens, "V - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("vd4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(vd4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            phi = tde4.getLensLDAdjustableParameter(lens, "Phi - Cylindric Direction", F, focus)
                            f.write(   '    param = rman.getChild("phi")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(phi, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            B = tde4.getLensLDAdjustableParameter(lens, "B - Cylindric Bending", F, focus)
                            f.write(   '    param = rman.getChild("B")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(B, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                        
                        elif model == '3DE4 Anamorphic - Standard, Degree 4':
                            cx02d2 = tde4.getLensLDAdjustableParameter(lens, "Cx02 - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("cx02d2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx02d2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy02d2 = tde4.getLensLDAdjustableParameter(lens, "Cy02 - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("cy02d2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy02d2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx22d2 = tde4.getLensLDAdjustableParameter(lens, "Cx22 - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("cx22d2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx22d2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy22d2 = tde4.getLensLDAdjustableParameter(lens, "Cy22 - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("cy22d2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy22d2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx04d4 = tde4.getLensLDAdjustableParameter(lens, "Cx04 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cx04d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx04d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy04d4 = tde4.getLensLDAdjustableParameter(lens, "Cy04 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cy04d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy04d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx24d4 = tde4.getLensLDAdjustableParameter(lens, "Cx24 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cx24d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx24d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy24d4 = tde4.getLensLDAdjustableParameter(lens, "Cy24 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cy24d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy24d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx44d4 = tde4.getLensLDAdjustableParameter(lens, "Cx44 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cx44d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx44d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy44d4 = tde4.getLensLDAdjustableParameter(lens, "Cy44 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cy44d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy44d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            lensRot = tde4.getLensLDAdjustableParameter(lens, "Lens Rotation", F, focus)
                            f.write(   '    param = rman.getChild("rot")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(lensRot, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            sqX = tde4.getLensLDAdjustableParameter(lens, "Squeeze-X", F, focus)
                            f.write(   '    param = rman.getChild("sqx")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(sqX, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            sqY = tde4.getLensLDAdjustableParameter(lens, "Squeeze-Y", F, focus)
                            f.write(   '    param = rman.getChild("sqy")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(sqY, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                        
                        
                        
                        elif model == '3DE4 Radial - Fisheye, Degree 8':
                            dd2 = tde4.getLensLDAdjustableParameter(lens, "Distortion - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("dd2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(dd2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            qd4 = tde4.getLensLDAdjustableParameter(lens, "Quartic Distortion - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("qd4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(qd4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            d6 = tde4.getLensLDAdjustableParameter(lens, "Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            d8 = tde4.getLensLDAdjustableParameter(lens, "Degree 8", F, focus)
                            f.write(   '    param = rman.getChild("d8")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(d8, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                        
                        
                        
                        elif model == '3DE4 Anamorphic, Degree 6':
                            cx02d2 = tde4.getLensLDAdjustableParameter(lens, "Cx02 - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("cx02d2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx02d2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy02d2 = tde4.getLensLDAdjustableParameter(lens, "Cy02 - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("cy02d2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy02d2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx22d2 = tde4.getLensLDAdjustableParameter(lens, "Cx22 - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("cx22d2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx22d2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy22d2 = tde4.getLensLDAdjustableParameter(lens, "Cy22 - Degree 2", F, focus)
                            f.write(   '    param = rman.getChild("cy22d2")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy22d2, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx04d4 = tde4.getLensLDAdjustableParameter(lens, "Cx04 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cx04d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx04d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy04d4 = tde4.getLensLDAdjustableParameter(lens, "Cy04 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cy04d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy04d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx24d4 = tde4.getLensLDAdjustableParameter(lens, "Cx24 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cx24d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx24d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy24d4 = tde4.getLensLDAdjustableParameter(lens, "Cy24 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cy24d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy24d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx44d4 = tde4.getLensLDAdjustableParameter(lens, "Cx44 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cx44d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx44d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy44d4 = tde4.getLensLDAdjustableParameter(lens, "Cy44 - Degree 4", F, focus)
                            f.write(   '    param = rman.getChild("cy44d4")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy44d4, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx06d6 = tde4.getLensLDAdjustableParameter(lens, "Cx06 - Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("cx06d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx06d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy06d6 = tde4.getLensLDAdjustableParameter(lens, "Cy06 - Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("cy06d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy06d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx26d6 = tde4.getLensLDAdjustableParameter(lens, "Cx26 - Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("cx26d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx26d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy26d6 = tde4.getLensLDAdjustableParameter(lens, "Cy26 - Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("cy26d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy26d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx46d6 = tde4.getLensLDAdjustableParameter(lens, "Cx46 - Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("cx46d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx46d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy46d6 = tde4.getLensLDAdjustableParameter(lens, "Cy46 - Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("cy46d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy46d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cx66d6 = tde4.getLensLDAdjustableParameter(lens, "Cx66 - Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("cx66d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cx66d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                            cy66d6 = tde4.getLensLDAdjustableParameter(lens, "Cy66 - Degree 6", F, focus)
                            f.write(   '    param = rman.getChild("cy66d6")\n')
                            f.write(   '    param.getChild("value").setKey(%s)\n'%(frame+frame0) )     
                            f.write(   '    param.getChild("value").setValue(%s, %s)\n'%(cy66d6, frame+frame0) ) 
                            f.write(   '    setLocal(param)\n\n')
                        
                        frame   += 1
                                
            #
            # global (scene node) transformation...
            '''
            # Katana doesn't support camera transformation through a Transform3D node
            # but only the camera local transform params are respected
            ## No need to calculate global scene transformation, but recommending getting cam into katana as Alembic ##
            p3d	= tde4.getScenePosition3D()
            p3d	= convertZup(p3d,yup)
            r3d	= tde4.getSceneRotation3D()
            rot	= convertToAngles(r3d)
            s	= tde4.getSceneScale3D()
            '''

            f.write("\n")
            f.close()
            
            tde4.postQuestionRequester("Export Nothing Real bs_tdeLens for Pixar RMS KATANA...","Project successfully exported.\nPlease, make sure bs_tdeLens for Katana is already installed,\nthen run the script when PrmanGlobalStatements you want to configure is selected.","Ok")
        else:
            tde4.postQuestionRequester("Export Nothing Real bs_tdeLens for Pixar RMS KATANA...","Error, couldn't open file.","Ok")

